use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Default)]
pub struct ElementId(Uuid);
impl ElementId {
    pub fn new() -> Self {
        ElementId(Uuid::new_v4())
    }

    pub fn bytes(&self) -> &[u8; 16] {
        self.0.as_bytes()
    }
}

impl ToString for ElementId {
    fn to_string(&self) -> String {
        self.0.to_string()
    }
}

impl From<&ElementId> for String {
    fn from(value: &ElementId) -> Self {
        value.0.to_string()
    }
}

impl TryFrom<&str> for ElementId {
    type Error = serde_json::Error;
    fn try_from(value: &str) -> Result<Self, Self::Error> {
        serde_json::from_str(value)
    }
}

impl From<Uuid> for ElementId {
    fn from(value: Uuid) -> Self {
        ElementId {0: value}
    }
}

impl From<[u8; 16]> for ElementId {
    fn from(value: [u8; 16]) -> Self {
        ElementId {0: Uuid::from_bytes(value)}
    }
}